#!/usr/bin/perl
use lib '/home/martin/ethereal/mgmt';

#################################################################################
# Created       : Martin Foster
# Modified      : 23-Nov-2004
#################################################################################
#
# Queue - Script designed to create an easier way to deal with dependencies, file
#         locks and concurrency issues when dealing with maintenance.
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use Net::Ping;						# Check on the neighbour
use strict;						# Strict variable enforcement

#################################################################################
# Data Members
#################################################################################

#
# User defined
#

# Primary maintenance script
my $base = '/home/martin/ethereal/mgmt/commandline/erealms.pl';

# Lock vile
my $lock = '/home/martin/tmp/erealms.lock';

# Standard hosts
my $db = 'localhost';


#
# System variables
#

# Verbose display
my $verbose = 1;

# Command
my $command = 'info';

# Filehandles
my $fh;
my $err;
my $io;

# Argument
my $arg;


#################################################################################
# User specific variables
#################################################################################

# Apache
my $apache = '/usr/sbin/apachectl';

# Location of mailing list manager;
my $mailman = '/home/martin/sync_members';

# Location of ssh
my $ssh = '/usr/bin/ssh';


#################################################################################
# Queues
#################################################################################
#
# For simplicity queues are defined as an array of hashes.   Allowing for the
# process to be performed in a similiar manner to a queue while granting the
# adminisrator of the system enough control to fine tune things like comments,
# forced pauses and even the calling of external programs.
#
# The hash element itself makes use of a consistent set of keys as to simplify
# matters overall.   The format is as follows:
#
# BUILTIN  => Builtin function name used with the standard script;
# EXTERNAL => External call to a command such as apache or db backups;
# COMMENT  => Comment to be displayed in verbose mode;
# SLEEP    => Forced period of sleep between queued tasks;
# PING     => Prompts the system to ping and verify a host is up;
#
# There are multiple queues called from the interface based on the times they
# are themselves called.   A list one would expect to see is below:
#
# QUARTER  => Run four times an hour;
# HOURLY   => Run on an hourly basis;
# DAILY    => Once a day;
# WEEKLY   => Once a week;
# MONTHLY  => Once a month;
#
# All queues are called from an external task such as CROND(5) which will have
# the ability to be fine tuned.   This script serves more as a wrapper allowing
# for enhanced safety when calling scripts that can potentially result in a
# deadlock.
#
################################################################################# 

# Interval : */20
# Cron     :

my $quarter = [
  {
    COMMENT  => "Generation:"
  },

  {
    BUILTIN  => "-forcast",
    COMMENT  => "  Updating weather forecasts:"
  },

  {
    BUILTIN  => "-banner",
    COMMENT  => "  Cycling banner ads in realm:"
  },

  {
    BUILTIN  => "-rss",
    COMMENT  => "  Update to RSS feeds:"
  },

  {
    COMMENT  => "Maintenance:"
  },

  {
    BUILTIN  => "-purge",
    COMMENT  => "  Removing old parameters from database:",
    SLEEP    => 0
  }
];

 
# Interval : 
# Cron     :

my $hour = [
  {
    COMMENT  => "Maintenance:"
  },

  {
    BUILTIN  => "-post 1",
    COMMENT  => "  Removing old posts from the database:",
    SLEEP    => 40
  },

  {
    BUILTIN  => "-expire audit",
    COMMENT  => "  Removing old audit entries from the database:",
    SLEEP    => 0
  }
 ];

 
# Interval : 
# Cron     :

my $day = [
  {
    COMMENT  => "\nStatistics:"
  },

  {
    BUILTIN  => "-notice \"Daily maintenance in process, expect slowdowns!\"",
    COMMENT  => "  Notify users of process beginning:",
    SLEEP    => 45
  },
  
#  {
#    BUILTIN  => "-realm",
#    COMMENT  => "  Realm level statistics:",
#    SLEEP    => 25
#  },

  {
    BUILTIN  => "-summary",
    COMMENT  => "  System level summaries:",
    SLEEP    => 25
  },

  {
    COMMENT  => "\nMaintenance:"
  },

  {
    BUILTIN  => "-expire",
    COMMENT  => "  Expire out of date content:",
    SLEEP    => 45
  },

  {
    COMMENT  => "\nMailing lists:"
  },

  {
    EXTERNAL => "$base ethereal -list -e | $mailman -w=no -a=no -f - members_ethereal-realms.org",
    COMMENT  => "  Ethereal's Members List:"
  },

  {
    EXTERNAL => "$base ethereal -list -s | $mailman -w=no -a=no -f - hosts_ethereal-realms.org",
    COMMENT  => "  Ethereal's Hosts List:"
  },

  {
    EXTERNAL => "$base ethereal -list -v | $mailman -w=no -a=no -f - contributors_ethereal-realms.org",
    COMMENT  => "  Ethereal's Contributors List:"
  },
 ];

 
# Interval : 
# Cron     :

my $week = [
  {
    COMMENT  => "\nStatistics:"
  },

  {
    BUILTIN  => "-notice \"Weekly maintenance in process, expect a halt in service shortly!\"",
    COMMENT  => "  Notify users of process beginning:",
    SLEEP    => 45
  },
  
  {
    BUILTIN  => "-ban",
    COMMENT  => "  Generating banned user reports:",
    SLEEP    => 25
  },

  {
    COMMENT  => "\nMaintenance:"
  },

  {
    BUILTIN  => "-stream \"Database level maintenance is being conducted, webserver will be going down!\"",
    COMMENT  => "  Notify users dropping webserver:",
    SLEEP    => 45
  },

  {
    BUILTIN  => "-vacuum complete",
    COMMENT  => "  In depth database maintenance",
    SLEEP    => 10
  },
 ];

 
# Interval : 
# Cron     :

my $month = [
  {
    COMMENT  => "\nMaintenance:"
  },

  {
    BUILTIN  => "-notice \"Monthly maintenance in process, minor delays expected!\"",
    COMMENT  => "  Notify users of process beginning:",
    SLEEP    => 45
  },
  
  {
    BUILTIN  => "-reset",
    COMMENT  => "  Reset cumulative statistics:",
    SLEEP    => 0
  }
 ];


#################################################################################
# Program Area
#################################################################################

	# Handle parameters
	# Determine behaviour
	while ($arg = shift(@ARGV))
	{
		# Disable verbose mode
		$verbose = 0 if ($arg =~ /-q/);

		# Command change
		$command = $arg if ($arg =~ /(quarter|hour|day|week|month)/);
	}

	# Script flow
	# The onstar of scripts
	if ($command eq 'info')
	{
		# Display useage line
		print "Use: ./queue.pl [-q] {quarter|hour|day|week|month}\n\n";	
	}

	# Normal operation
	else
	{
		# Lock file
		($fh, $io, $err) = GetLock($lock, $verbose);

		# Queue
		# Load proper queue
		Queue($base, $db, $quarter) if ($command =~ /quarter/);	
		Queue($base, $db, $hour)    if ($command =~ /hour/);	
		Queue($base, $db, $day)     if ($command =~ /day/);	
		Queue($base, $db, $week)    if ($command =~ /week/);	
		Queue($base, $db, $month)   if ($command =~ /month/);	

		# Release lock
		KillLock($lock, $fh, $io, $err);
	}

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Get Lock
#
# One of the core compoents of this queue is the ability to prevent other
# instances of itself from running concurrently, removing the possibility of
# a deadlocked process due to timing conflicts.
#
# Since this will create a file lock, the system will also redirect output
# when a lock is established.

sub GetLock
{
	#####################
	# Data members

	my $lock    = shift;					# Lock file
	my $verbose = shift;					# Verbose setting

	my $count   = 0;					# Counter
	my $sleep   = 1;					# Sleep timer
	my $limit   = 10;					# Give up period


	#####################
	# Program area

	# Lock file exists
	while (-f $lock)
	{
		# Adjust sleep timer
		$sleep += $sleep;
		$count += 1;	

		# Over limit
		die('ERROR: Failed to obtain a lock in time!') if ($count > $limit);

		# Regular wait
		sleep $sleep;
		
	}

	# File handle
	# Create file lock
	open(FILE, '>', $lock) or die("ERROR: Unable to obtain file lock: $!");

	# Drop in process list
	print FILE "$$\n\n";
	

	# Redirection
	# Copy
	open(OLDOUT, ">&STDOUT");
	open(OLDERR, ">&STDERR");
	

	
	# Redirect verbosity
	#*STDIN  = *FILE if ($verbose != 1);
	open(STDOUT, ">&FILE") or die("ERROR: Cannot redirect stdout") if ($verbose != 1);

	# Standard redirection
	#*STDERR = *FILE;
	open(STDERR, ">&FILE") or die("ERROR: Cannot redirect sterr");


	# Return filehandle copy
	return *FILE, *OLDOUT, *OLDERR;
}


#####################
# Kill Lock
#
# Required process of the system in order to free up the lock and ensure that a
# new process can take over and get the job done as necessary.

sub KillLock
{
	#####################
	# Data member

	my $lock    = shift;					# Lock file
	my $fh      = shift;					# Provided filehandle
	my $io      = shift;					# Default STDIO
	my $err     = shift;					# Default STDERR


	#####################
	# Program area

	# Return stdio and stderr
	open(STDOUT, $io);
	open(STDERR, $err);
	
	
	# Close filehandle
	close($fh) or die("ERROR: Unable to release filehandle: $!");

	# Remove file
	unlink($lock) or die("ERROR: Unable to remove file lock: $!");
}


#####################
# Queue
#
# Performs the bulk of the processing necessary to complete the tasks in
# the queue itself.

sub Queue
{
	#####################
	# Data members

	my $base   = shift;					# Base code
	my $db     = shift;					# Database server
	my $ref    = shift;					# Referenced array

	my $hash;						# Singular hash
	my $op;							# Operations
	my $ping;						# Poke a friend
	my $sleep;						# Narcolyptic

	my %hash;						# Holding hash

	
	#####################
	# Program area

	# Actually what we want
	return unless (ref($ref) eq 'ARRAY');


	# Initialize ping handle
	$ping = new Net::Ping();

	
	# While there are tasks
	while ($hash = shift(@{$ref}))
	{
		# Skip
		next unless (ref($hash) eq 'HASH');


		# Adjust sleep patterns
		# Given a healthy spacing no matter
		$sleep = (defined($hash->{'SLEEP'})) ? $hash->{'SLEEP'} : 2;


		# Handle operations
		# Comments
		print "\n" . $hash->{'COMMENT'} . "\n" if (defined($hash->{'COMMENT'}));

		# Built in operations
		if (defined($hash->{'BUILTIN'}))
		{
			# Host is alive
			# Database activity is a must
			if ($ping->ping($db))
			{
				# Evaluate command
				system($base . ' ' .  $hash->{'BUILTIN'}) == 0
				 or die "Unable to fork to program $?";

				# Snooze a bit
				sleep($sleep);
			}

			# Next pass
			next;
		}

		# Check for life signs
		if (defined($hash->{'PING'}))
		{
			# Do a quick check and pass
			next unless ($ping->ping($hash->{'PING'}));
		}
		
		# External commands
		if (defined($hash->{'EXTERNAL'}))
		{
			# Evaluate command
			eval
			{
				# Evaluate command
				system($hash->{'EXTERNAL'}) == 0
				 or die "Unable to fork to program $?";

				# Snooze a bit
				sleep($sleep);
			};

			print $! if ($@);
		}
	}
}
